/*
** FCSTUNIV.g
**
** Purpose : compute forecasts from univariate nonlinear time series models
**           using Monte Carlo or bootstrap method
**
** Format  : {FCST,realv}=FCSTUNIV(hist,yi,N,R,&model,&dist,&tr_res);
**
** Input   : hist     : (1xp) vector, history of time series
**           yi       : (mx1), colum indices to be stored
**           N        : scalar, maximum forecast horizon
**           R        : scalar, number of replications to use in bootstrap forecasting
**                        If R=0, naive forecasts are computed    
**           &model   : pointer to procedure to compte output of model
**           &dist    : pointer to procedure to generate STANDARDIZED residuals
**           &tr_res  : pointer to procedure to transform standardized residuals
**
** Output  : FCST     : (Nxm) matrix, containing point forecasts
**           realv    : (Nxm*R) matrix, containing individual realizations used in 
**                         constructing bootstrap/Monte Carlo forecasts
**
** Globals :
**
** Remarks : 1) The procedure &model should have two inputs:
**                (pxR) matrix of histories and 
**                (1xR) matrix of shocks, 
**           2) The single output of &model should be a (pxR) matrix of "new" histories
**           3) The procedure &dist should give an (R x k) matrix of standardized residuals
**              as the only output, where R and k are the two inputs
**
**           Any parameters used by these procedures should be defined as globals   
**
** Written by Dick van Dijk
**
** Last revision on 22 December 1999
**
*/
proc (2)= FCSTUNIV(hist,yi,N,R,&model,&dist,&tr_res);
  local m,realv,histv,shocv,i,j,fcst,st_shock,deter;
  local model: proc;
  local dist: proc;
  local tr_res: proc;

if (R==0); /* naive forecasting */
  R=1; deter=1;
else;
  deter=0;
endif;

m=rows(yi);

/* initialization */
realv=zeros(R*m,N);
fcst=zeros(N,m);

histv=hist.*ones(R,1);
/* iterate over horizon */
i=0;
do until (i==N);
  i=i+1;

  if not deter;
    /* generate standardized shocks */
    st_shock=dist(R,1);
    /* transform shocks */
    shocv=tr_res(histv,st_shock);
    /* compute new histories */
  else;
    shocv=0;
  endif;
  histv=model(histv,shocv); 
  /* extract forecasts */
  j=0;
  do until (j==m);
    j=j+1;
    realv[(j-1)*R+1:j*R,i]=histv[.,yi[j]];
    fcst[i,j]=meanc(realv[.,i]);
  endo;
endo;
retp(fcst,realv');
endp;
