/*
** LSTREST.e
**
** purpose : Estimation of logistic smooth transition regression (LSTR) model 
**           by nonlinear least squares (quasi-maximum likelihood)
**
** output  : Plots of actual series and fitted values, residuals, transition function
**           against time and transition variable.
**           File containing parameter estimates, s.e.'s and t-values, misspecification
**           tests for residual serial correlation, constancy of parameters
**           and remaining nonlinearity, and other diagnostistics.
**
** remarks : This program uses the CML library
**
** Last revision on 26 October 2000
** Written by Dick van Dijk
**
*/

new; cls;
trap 1;

library cml,pgraph;
cmlset; graphset;

#include kernel.prc;

format /rd 12,6;
__maxvec=50000;

p_max=18;                               /* Maximum autoregressive lag */
u_dif=1;                                /* u_dif=1 if model is for first differences, 
                                           u_dif=0 if model is for levels */
i_lev=1;                                /* i_lev=1 if lagged level is to be included, 
                                           i_lev=0 if not (NB only relevant if u_dif=1)*/

l_dif=12;                               /* value of "long difference" which is used as transition variable */
max_yd=6;                               /* maximum delay of transition variable */

/*
** Specify sample period of data in files
*/
n_fy=48;                                /* Year of first observation */
n_fm=1;                                 /* Month of first observation */
n_ly=99;                                /* Year of last observation */
n_lm=12;                                /* Month of last observation */
d_nobs=12*(n_ly-n_fy)+n_lm-n_fm+1;      /* Number of observations */

/*
** Specify sample period for which model is to be estimated
*/
e_fy=68; /* year of first observation */
e_fm=6;  /* month of first observation */
e_ly=89; /* year of last observation */
e_lm=12; /* month of last observation */

/* Create monthly dummies */
m_dum=ones(n_ly-n_fy+1,1).*.eye(12);
m_dum=m_dum[n_fm:n_fm+d_nobs-1,.];

/* load raw data */
load lhmu[]=lhmunsa.dat; /* number of unemployed */
load lhmc[]=lhmcnsa.dat; /* labor force */
/* compute unemployment rate */
y_raw=(lhmu./lhmc)*100;

e_nobs=12*(e_ly-e_fy)+e_lm-e_fm+1; /* Number of observations in estimation period */

tdat=packr(y_raw[12*(e_fy-n_fy)+e_fm-n_fm+1:12*(e_fy-n_fy)+e_fm-n_fm+e_nobs]~
           m_dum[12*(e_fy-n_fy)+e_fm-n_fm+1:12*(e_fy-n_fy)+e_fm-n_fm+e_nobs,.]);
y_lev=tdat[.,1];
m_dum=tdat[.,2:13];
T=rows(y_lev);

if u_dif; /* model for first differences */
  y_dif=(y_lev[2:T]-y_lev[1:T-1]); 
  m_dum=m_dum[2:T,.];
  T=T-1;  
  if i_lev; /* include lagged level term */
    deter=ones(T,1)~m_dum~y_lev[1:T];
  else;
    deter=ones(T,1)~m_dum;
    endif;
else; /* model for levels */
  y_dif=y_lev;
  deter=ones(T,1)~m_dum~seqa(1,1,T);
endif;

/* Construct data matrix */
_dat=trimr(lagn(y_dif,seqa(0,1,p_max+1)),p_max,0);
j=0;
do until (j==max_yd);
  j=j+1;
  if u_dif;
    _dat=_dat~sumc(_dat[.,1+j:1+j+l_dif-1]');
  else;
    _dat=_dat~(_dat[.,1+j]-_dat[.,1+j+l_dif]);
  endif;
endo;
_dat=ones(T-p_max,1)~m_dum[p_max+1:T,.]~seqa(1/(T-p_max),1/(T-p_max),T-p_max)~_dat;
if i_lev;
  _dat=_dat~y_lev[p_max+1:T];
endif;
T=rows(_dat);

/* get names for variables in _dat matrix */
names=GETNAMES(u_dif,i_lev,p_max,max_yd);

/*
** Specify column index of regressand (_yi), 
** indices of regressors which enter linearly (_xi0), 
** indices of regressors which enter in "lower" regime G=0 (_xi1), 
** indices of regressors which enter in "upper" regime G=1 (_xi2), 
** and index of transition variable (_xtri),
*/
_yi=15;
_xi0=1|2|3|4|5|6|7|8|9|10|11;
_xi1=(14+1+p_max+max_yd+1)|16|21|23|25|28|30;
_xi2=(14+1+p_max+max_yd+1)|16|17|22|28|29|30;
_xtri=(14+1+p_max+1);

if not scalmiss(_xi0);
  nk_x0=rows(_xi0);
else;
  nk_x0=0;
endif;
if not scalmiss(_xi1);
  nk_x1=rows(_xi1);
else;
  nk_x1=0;
endif;
if not scalmiss(_xi2);
  nk_x2=rows(_xi2);
else;
  nk_x2=0;
endif;

_xi=_xi0|_xi1|_xi2;
nk_xi=nk_x0|nk_x1|nk_x2;
nk_x=sumc(nk_xi);
nk_xii=0|cumsumc(nk_xi);

/* column indices of constant, dummies and lags */
_xic=indexcat(_xi,0|1.5);
_xidum=indexcat(_xi,0|12.5);
_xilag=indexcat(_xi,13.5|(14+1+p_max+max_yd+2));

_thrzero=0; /* set _thrzero=1 to restrict location parameter c to 0 */

/* compute scaling factor for transition function */
_sc=stdc(_dat[.,_xtri]);

/* set bounds on parameters such that gamma is ensured to be positive */
bounds={-1E3 1E3};
_cml_Bounds=ones(2,2).*bounds;
_cml_Bounds[1,.] = ln((5E-2~5E2));

/*
** specify optimization algorithm, type of covariance matrix of parameters,
** method for computing numerical gradient or procedure to compute gradient,
** convergence tolerance for gradient, maximum number of iterations
*/
_cml_Algorithm=3;          /* 3=Newton-Raphson optimization */
_cml_CovPar=0;             /* 3=heteroskedastic-consistent cov matrix */
_cml_GradProc=&g_ssr_LSTR; /* procedure containing analytical gradient */
_cml_DirTol=1E-5;          /* convergence tolerance for gradient */
_cml_MaxIters=1000;        /* max number of iterations */
__output=2;                /* This causes extra output to be displayed */

/*
** specify inequality restraints to be satisfied by c if it is allowed to
** be nonzero (c is restricted to lie between perc[1]*T and perc[2]*T percentiles
** of transition variable
*/
perc={.025,.975};
s_tr=sortc(_dat[.,_xtri],1);
if (not _thrzero); /* c not restricted */
  _cml_C=zeros(2,1)~seqm(1,-1,2);
  _cml_D=s_tr[ceil(perc[1]*T)]|-s_tr[floor(perc[2]*T)];
else; /* c restricted to 0 */
  _cml_C={};
  _cml_D={};
endif;

/* determine starting values by grid search over gamma and c */
param_0=zeros(2-_thrzero,1);
_param_l=zeros(nk_x,1);
param_0=GETSTART(param_0);
param=param_0;

output file="svnls.out" reset; screen off;
  param;
output off; screen on;

{param,fparam,gparam,cparam,retcode}=cml(_dat,0,&ssr_LSTR,param);

/* Compute fitted values, residuals, and standard errors */
fit=skel_LSTR(param,_dat);
resid=_dat[.,_yi]-fit;
sigML=(resid'resid)/T;
sigLS=(resid'resid)/(T-nk_x-2);

{vcov,hccvcov}=vcv_LSTR(param,_dat,resid);
gradc=grd_LSTR(param,_dat,resid);
ster=sqrt(diag(vcov));
hccster=sqrt(diag(hccvcov));

/* compute misspecification tests */
/* _xtre contains indices of transition variables in tests of no remaining nonlinearity */
_xtre=seqa(14+1+p_max+1,1,max_yd); 
namese=names[_xtre];
k_xtre=6;
q_max=12; /* maximum order of residual serial correlation to test against */
q_v=seqa(1,1,q_max);
ARCHq=4; /* maximum ARCH order to test against */

/* 
** gradient w.r.t. intercept should be omitted in misspecification tests to
** avoid singular moment matrices in auxiliary regressions
*/
except=zeros(max_yd,nk_x);
except[1:4,_xic]=ones(4,rows(_xic));

{LM_SIc,pv_SIc,LM_SIf,pv_SIf,LM_RNLcd,pv_RNLcd,LM_RNLfd,pv_RNLfd,
 LM_Ccd,pv_Ccd,LM_Cfd,pv_Cfd,LM_NLvc,pv_NLvc,LM_NLvf,pv_NLvf,LM_Cvc,pv_Cvc,LM_Cvf,pv_Cvf}=
 DGNLSTR(resid,param,q_max,_xtre,_xidum,_xidum,except[.,_xidum]);

{LM_SIc,pv_SIc,LM_SIf,pv_SIf,LM_RNLcl,pv_RNLcl,LM_RNLfl,pv_RNLfl,
 LM_Ccl,pv_Ccl,LM_Cfl,pv_Cfl,LM_NLvc,pv_NLvc,LM_NLvf,pv_NLvf,LM_Cvc,pv_Cvc,LM_Cvf,pv_Cvf}=
 DGNLSTR(resid,param,q_max,_xtre,_xic|_xilag,_xic|_xilag,except[.,_xic|_xilag]);

{LM_SIc,pv_SIc,LM_SIf,pv_SIf,LM_RNLc,pv_RNLc,LM_RNLf,pv_RNLf,
 LM_Cc,pv_Cc,LM_Cf,pv_Cf,LM_NLvc,pv_NLvc,LM_NLvf,pv_NLvf,LM_Cvc,pv_Cvc,LM_Cvf,pv_Cvf}=
 DGNLSTR(resid,param,q_max,_xtre,seqa(1,1,nk_x),seqa(1,1,nk_x),except);

{meanSTR,minSTR,maxSTR,medSTR,stdevSTR,madSTR,skewSTR,pskewSTR,kurtSTR,pkurtSTR,
  JBSTR,pJBSTR,corrSTR,LBSTR,pLBSTR,LMSTR,pLMSTR,ARCHSTR,pARCHSTR,AICSTR,BICSTR,HQSTR } =
  DGNRESID(resid,nk_x+2,nk_x+14,ARCHq,gradc);

/* Estimate linear model */
_xil=1|2|3|4|5|6|8|9|10|11|(14+1+p_max+max_yd+1)|16|17|18|21|23|26|30;
{ beta,seb,hcseb }=OLSHC(_dat[.,_yi],_dat[.,_xil]);
residl=_dat[.,_yi]-_dat[.,_xil]*beta;

ARCHq=4;
{ meanL,minL,maxL,medL,stdevL,madL,skewL,pskewL,kurtL,pkurtL,
  JBL,pJBL,corrL,LBL,pLBL,LML,pLML,ARCHL,pARCHL,AICL,BICL,HQL } =
DGNRESID(residl,rows(_xil),rows(_xil)+14,ARCHq,_dat[.,_xil]);

/* prepare for output */
param[1]=exp(param[1]);
param=_param_l|param;
if _thrzero; param=param|0; endif;
tr_fun=LOGF(_dat[.,_xtri],_sc,param[nk_x+1],param[nk_x+2]);

/* generate output files and graphs */
THE_OUTPUT;
THE_FIGURES(fit,resid,tr_fun);

end; /* end of main program */


/*
** GETSTART.g
**
** determine starting values by performing two-dimensional grid search over
** gamma and c (only when param_0 is equal to 0)
*/
proc (1)=GETSTART(param_0);
  local gam_vec,ng,step,c_vec,nc,i,j,fun,x_mat,xx_i,param_lt,ssr_t,ssr_max;

if (param_0==0);
  gam_vec=2.5|5|10|25;
  ng=rows(gam_vec);
  if (not _thrzero[1]);
    step=(s_tr[floor(perc[2]*T)]-s_tr[ceil(perc[1]*T)])/10;
    c_vec=seqa(s_tr[ceil(perc[1]*T)]+.5*step,step,10);
  else;
    c_vec=0;
  endif;
  nc=rows(c_vec);
  
  ssr_max=-1E10;

  i=0;
  do until (i==ng);
    i=i+1;
    j=0;
    do until (j==nc);
      j=j+1;

      fun=LOGF(_dat[.,_xtri],_sc,gam_vec[i],c_vec[j]);
      x_mat=GETXXX(_dat,fun);
      xx_i=invpd(moment(x_mat,0));
      if not scalmiss(xx_i);
        param_lt=xx_i*((x_mat)'_dat[.,_yi]);
        ssr_t=-sumc((_dat[.,_yi]-(x_mat)*param_lt).^2);
        if (ssr_t GE ssr_max);
          ssr_max=ssr_t;
          _param_l=param_lt;
          param_0=ln(gam_vec[i]);
          if not _thrzero;
            param_0=param_0|c_vec[j];
          endif;
        endif;
      endif;
    endo;
  endo;
else;
  fun=LOGF(_dat[.,_xtri],_sc,param_0[1],param_0[2]);
  x_mat=GETXXX(_dat,fun);
  xx_i=invpd(moment(x_mat,0));
  if not scalmiss(xx_i);
    param_lt=xx_i*((x_mat)'_dat[.,_yi]);
    ssr_t=-sumc((_dat[.,_yi]-(x_mat)*param_lt).^2);
    _param_l=param_lt;
  endif;
endif;

retp(param_0);
endp;


/*
**  ssr_LSTR.g
**
**  compute minus sum of squared residuals of two-regime LSTR model
**
*/
proc ssr_LSTR(theta,dat);

retp(-((dat[.,_yi]-skel_LSTR(theta,dat)).^2));
endp;


/*
** g_ssr_LSTR.g
**
** compute gradient of minus sum of squared residuals of LSTR model
*/
proc g_ssr_LSTR(theta,dat);

retp(2*(dat[.,_yi]-skel_LSTR(theta,dat))'*g_skel_LSTR(theta,dat));
endp;


/*
** skel_LSTR.g
**
** compute fitted value ('skeleton') for two-regime LSTR model
**
*/
proc skel_LSTR(theta,dat);
  local fun,x,xxi,y;

if _thrzero;
  theta=theta|0;
endif;

fun=LOGF(dat[.,_xtri],_sc,exp(theta[1]),theta[2]);
x=GETXXX(dat,fun);
xxi=invpd(x'x);
if not scalmiss(xxi);
  _param_l=xxi*(x'dat[.,_yi]);
endif;
y=x*_param_l;

retp(y);
endp;


/*
** g_skel_LSTR.g
**
** compute gradient vector of skeleton of two-regime LSTR model
**
*/
proc g_skel_LSTR(theta,dat);
  local T,fun,dfdg,dfdc,dgdx,dgam,dc;

T=rows(dat);
if _thrzero;
  theta=theta|0;
endif;
fun=LOGF(dat[.,_xtri],_sc,exp(theta[1]),theta[2]);

dfdg=exp(theta[1]).*(dat[.,_xtri]-theta[2]).*fun.*(1-fun)./_sc;
dfdc=-(exp(theta[1])./_sc).*ones(T,1).*fun.*(1-fun);

dgdx=0;
if (nk_x1 GT 0);
  dgdx=dgdx-dat[.,_xi1]*_param_l[nk_xii[1+1]+1:nk_xii[2+1]];
endif;
if (nk_x2 GT 0);
  dgdx=dgdx+dat[.,_xi2]*_param_l[nk_xii[2+1]+1:nk_xii[3+1]];
endif;

dgam=dgdx.*dfdg;
dc={};
if not _thrzero;
  dc=dgdx.*dfdc;
endif;

retp(dgam~dc);
endp;


/*
** grd_LSTR.g
**
** compute gradient vector of parameter estimates for two-regime LSTR model
**
*/
proc (1)=grd_LSTR(theta,dat,res);
    local dgdc,fun,dphi,grad;

dgdc=g_skel_LSTR(theta,dat);
dgdc[.,1]=dgdc[.,1]./exp(theta[1]);
if _thrzero;
  theta=theta|0;
endif;
fun=LOGF(dat[.,_xtri],_sc,exp(theta[1]),theta[2]);
dphi=GETXXX(dat,fun);
grad=dphi~dgdc;

retp(grad);
endp;


/*
** vcv_LSTR.g
**
** compute variance-covariance matrix of parameter estimates for two-regime LSTR model
**
** Both standard and (`White') heteroskedasticity-consistent variants are
** computed
*/
proc (2)=vcv_LSTR(theta,dat,res);
    local grad,ggi,geeg,vcov,hccvcov;

grad=grd_LSTR(theta,dat,res);
ggi=invpd(grad'*grad);
geeg=grad'*((res.^2).*grad);
vcov=meanc(res.^2).*ggi;
hccvcov=ggi*geeg*ggi;

retp(vcov,hccvcov);
endp;


/*
** LOGF.g
**
** compute the value of the logistic transition function
**
*/
proc LOGF(x_trans,sc,gam,c);

retp(1./(1+exp(-(gam./sc).*(x_trans-c))));
endp;


/*
** GETXXX.g
**
** construct matrix with regressors in LSTR model 
**
*/
proc GETXXX(dat,fun);
 local x;

x={};
if (nk_x0 GT 0);
  x=x~dat[.,_xi0];
endif;
if (nk_x1 GT 0);
  x=x~(dat[.,_xi1].*(1-fun));
endif;
if (nk_x2 GT 0);
  x=x~(dat[.,_xi2].*fun);
endif;

retp(x);
endp;


/*
** OLSHC.g
**
** compute OLS regression with heteroskedasticity consistent standard errors
**
*/
proc (3) = olshc(y,x);
    local b,xxi,e,xeex,cvm,se,hcse;

b=y/x;
xxi=inv(x'*x);
e=y-x*b;
xeex=x'*((e^2).*x);
cvm=xxi*xeex*xxi;
se=sqrt(meanc(e^2).*diag(xxi));
hcse=real(sqrt(diag(cvm)));

retp(b,se,hcse);
endp;


/*
** GETNAMES.g
**
** construct names of variables in _dat matrix
**
*/
proc(1)=GETNAMES(u_dif,i_lev,p_max,max_yd);
  local names,namej,l,pn;

names="const"|"M1"|"M2"|"M3"|"M4"|"M5"|"M6"|"M7"|"M8"|"M9"|"M10"|"M11"|"M12"|"trend";
if (u_dif);
  names=names|"dy(t)";
else;
  names=names|"y(t)";
endif;
l=0;
do until (l==p_max);
  l=l+1;
  if l<10; pn=ftocv(l,1,0); else; pn=ftocv(l,2,0); endif;
  if u_dif;
    namej="" $+ "dy(t-" $+ pn $+ ")";
  else;
    namej="" $+ "y(t-" $+ pn $+ ")";
  endif;
  names=names|namej;
endo;
l=0;
do until (l==max_yd);
  l=l+1;
  if l<10; pn=ftocv(l,1,0); else; pn=ftocv(l,2,0); endif;
  namej="dy*(t-" $+ pn $+ ")";
  names=names|namej;
endo;
if (i_lev);
  names=names|"y(t-1)";
endif;

retp(names);
endp;


/* 
** THE_OUTPUT.g
**
** write parameter estimates, misspecification tests, etc, in output file
** 
*/
proc (0)=THE_OUTPUT;
  local namesn,i,ff,fname,z,m_r;

namesn={};
i=1;
do until (i==4);
  i=i+1;
  if (nk_xi[i-1] GT 0);
    namesn=namesn|names[_xi[nk_xii[i-1]+1:nk_xii[i]]];
  endif;
endo;
namesn=namesn|"gam"|"c";

format /rd 12,6;

ff= "#*.*lG"~14~8;
fname="lstrfit.out";
output file=^fname reset; screen off;
outwidth 120;

"Transition variable " $names[_xtri];"";
"Linear model";"";
call printfm("Variable"~"Estimate"~"StErr"~"HC-StErr"~"t-stat"~"HC-tstat",0~0~0~0~0~0,ff);"";
call printfm(names[_xil]~beta~seb~hcseb~(beta./(seb~hcseb)),0~1~1~1~1~1,ff);"";
"Sum of squared residuals " residl'residl;"";

"LSTR model";"";
call printfm("Variable"~"Estimate"~"StErr"~"HC-StErr"~"t-stat"~"HC-tstat",0~0~0~0~0~0,ff);"";
i=1;
do until (i==4);
  i=i+1;
  if (nk_xii[i] GT nk_xii[i-1]);
    z="_xi" $+ ftocv(i-2,1,0); z;
    z=namesn[nk_xii[i-1]+1:nk_xii[i]]~param[nk_xii[i-1]+1:nk_xii[i]]~
         ster[nk_xii[i-1]+1:nk_xii[i]]~hccster[nk_xii[i-1]+1:nk_xii[i]]~
         (param[nk_xii[i-1]+1:nk_xii[i]]./(ster[nk_xii[i-1]+1:nk_xii[i]]~
          hccster[nk_xii[i-1]+1:nk_xii[i]]));
    call printfm(z,0~1~1~1~1~1,ff);"";
  endif;
endo;
z=namesn[nk_xii[4]+1:nk_xii[4]+2]~param[nk_xii[4]+1:nk_xii[4]+2]~
         ster[nk_xii[4]+1:nk_xii[4]+2]~hccster[nk_xii[4]+1:nk_xii[4]+2]~
         (param[nk_xii[4]+1:nk_xii[4]+2]./(ster[nk_xii[4]+1:nk_xii[4]+2]~
          hccster[nk_xii[4]+1:nk_xii[4]+2]));
call printfm(z,0~1~1~1~1~1,ff);"";
"Sum of squared residuals " resid'resid;"";

format /rd 10,7;
"Diagnostic tests - F-variants";"";
"Test for residual serial correlation";
call printfm("Order"~"LS-Stat"~"p-value"~"W-Stat"~"p-value",zeros(1,5),ff);"";
call printfm(q_v~LM_SIf[.,1]~pv_SIf[.,1]~LM_SIf[.,2]~pv_SIf[.,2],ones(1,5),ff);"";
"Tests for constancy of parameters (all)";
call printfm("Test"~"LS-Stat"~"p-value"~"W-Stat"~"p-value",zeros(1,5),ff);"";
call printfm("LM_C1"~LM_Cf[1,1]~pv_Cf[1,1]~LM_Cf[1,2]~pv_Cf[1,2],0~ones(1,4),ff);"";
call printfm("LM_C2"~LM_Cf[2,1]~pv_Cf[2,1]~LM_Cf[2,2]~pv_Cf[2,2],0~ones(1,4),ff);"";
call printfm("LM_C3"~LM_Cf[3,1]~pv_Cf[3,1]~LM_Cf[3,2]~pv_Cf[3,2],0~ones(1,4),ff);"";
"";
"Tests for constancy of parameters (dummies)";
call printfm("Test"~"LS-Stat"~"p-value"~"W-Stat"~"p-value",zeros(1,5),ff);"";
call printfm("LM_C1"~LM_Cfd[1,1]~pv_Cfd[1,1]~LM_Cfd[1,2]~pv_Cfd[1,2],0~ones(1,4),ff);"";
call printfm("LM_C2"~LM_Cfd[2,1]~pv_Cfd[2,1]~LM_Cfd[2,2]~pv_Cfd[2,2],0~ones(1,4),ff);"";
call printfm("LM_C3"~LM_Cfd[3,1]~pv_Cfd[3,1]~LM_Cfd[3,2]~pv_Cfd[3,2],0~ones(1,4),ff);"";
"";
"Tests for constancy of parameters (lags)";
call printfm("Test"~"LS-Stat"~"p-value"~"W-Stat"~"p-value",zeros(1,5),ff);"";
call printfm("LM_C1"~LM_Cfl[1,1]~pv_Cfl[1,1]~LM_Cfl[1,2]~pv_Cfl[1,2],0~ones(1,4),ff);"";
call printfm("LM_C2"~LM_Cfl[2,1]~pv_Cfl[2,1]~LM_Cfl[2,2]~pv_Cfl[2,2],0~ones(1,4),ff);"";
call printfm("LM_C3"~LM_Cfl[3,1]~pv_Cfl[3,1]~LM_Cfl[3,2]~pv_Cfl[3,2],0~ones(1,4),ff);"";"";
"Tests for remaining nonlinearity (all) - quadratic terms";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_RNLf[1:max_yd,1]~pv_RNLf[1:max_yd,1]~
        LM_RNLf[1:max_yd,2]~pv_RNLf[1:max_yd,2],0~ones(1,4),ff);"";
"Tests for remaining nonlinearity (all) - cubic terms";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_RNLf[max_yd+1:2*max_yd,1]~pv_RNLf[max_yd+1:2*max_yd,1]~
        LM_RNLf[max_yd+1:2*max_yd,2]~pv_RNLf[max_yd+1:2*max_yd,2],0~ones(1,4),ff);"";
"Tests for remaining nonlinearity (all) - fourth powers";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_RNLf[2*max_yd+1:3*max_yd,1]~pv_RNLf[2*max_yd+1:3*max_yd,1]~
        LM_RNLf[2*max_yd+1:3*max_yd,2]~pv_RNLf[2*max_yd+1:3*max_yd,2],0~ones(1,4),ff);"";"";
"Tests for remaining nonlinearity (dummies) - quadratic terms";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_RNLfd[1:max_yd,1]~pv_RNLfd[1:max_yd,1]~
        LM_RNLfd[1:max_yd,2]~pv_RNLfd[1:max_yd,2],0~ones(1,4),ff);"";
"Tests for remaining nonlinearity (dummies) - cubic terms";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_RNLfd[max_yd+1:2*max_yd,1]~pv_RNLfd[max_yd+1:2*max_yd,1]~
        LM_RNLfd[max_yd+1:2*max_yd,2]~pv_RNLfd[max_yd+1:2*max_yd,2],0~ones(1,4),ff);"";
"Tests for remaining nonlinearity (dummies) - fourth powers";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_RNLfd[2*max_yd+1:3*max_yd,1]~pv_RNLfd[2*max_yd+1:3*max_yd,1]~
        LM_RNLfd[2*max_yd+1:3*max_yd,2]~pv_RNLfd[2*max_yd+1:3*max_yd,2],0~ones(1,4),ff);"";"";
"Tests for remaining nonlinearity (lags) - quadratic terms";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_RNLfl[1:max_yd,1]~pv_RNLfl[1:max_yd,1]~
        LM_RNLfl[1:max_yd,2]~pv_RNLfl[1:max_yd,2],0~ones(1,4),ff);"";
"Tests for remaining nonlinearity (lags) - cubic terms";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_RNLfl[max_yd+1:2*max_yd,1]~pv_RNLfl[max_yd+1:2*max_yd,1]~
        LM_RNLfl[max_yd+1:2*max_yd,2]~pv_RNLfl[max_yd+1:2*max_yd,2],0~ones(1,4),ff);"";
"Tests for remaining nonlinearity (lags) - fourth powers";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_RNLfl[2*max_yd+1:3*max_yd,1]~pv_RNLfl[2*max_yd+1:3*max_yd,1]~
        LM_RNLfl[2*max_yd+1:3*max_yd,2]~pv_RNLfl[2*max_yd+1:3*max_yd,2],0~ones(1,4),ff);"";"";
"Tests for constancy of variance";
call printfm("Test"~"LS-Stat"~"p-value"~"W-Stat"~"p-value",zeros(1,5),ff);"";
call printfm("LM_C1"~LM_Cvf[1,1]~pv_Cvf[1,1]~LM_Cvf[1,2]~pv_Cvf[1,2],0~ones(1,4),ff);"";
call printfm("LM_C2"~LM_Cvf[2,1]~pv_Cvf[2,1]~LM_Cvf[2,2]~pv_Cvf[2,2],0~ones(1,4),ff);"";
call printfm("LM_C3"~LM_Cvf[3,1]~pv_Cvf[3,1]~LM_Cvf[3,2]~pv_Cvf[3,2],0~ones(1,4),ff);"";"";
"Tests for nonlinearity in variance - quadratic terms";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_NLvf[1:max_yd,1]~pv_NLvf[1:max_yd,1]~
        LM_NLvf[1:max_yd,2]~pv_NLvf[1:max_yd,2],0~ones(1,4),ff);"";
"Tests for nonlinearity in variance - cubic terms";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_NLvf[max_yd+1:2*max_yd,1]~pv_NLvf[max_yd+1:2*max_yd,1]~
        LM_NLvf[max_yd+1:2*max_yd,2]~pv_NLvf[max_yd+1:2*max_yd,2],0~ones(1,4),ff);"";
"Tests for nonlinearity in variance - fourth powers";
call printfm("Tr. var"~"LS-Stat"~"p-value"~"W-Stat"~"p-val",zeros(1,5),ff);"";
call printfm(namese~LM_NLvf[2*max_yd+1:3*max_yd,1]~pv_NLvf[2*max_yd+1:3*max_yd,1]~
        LM_NLvf[2*max_yd+1:3*max_yd,2]~pv_NLvf[2*max_yd+1:3*max_yd,2],0~ones(1,4),ff);"";"";

"Residual diagnostics";
call printfm("Stat"~"LIN"~""~"LSTR",0~0~0~0,ff);"";
call printfm(""~"Stat"~"p-value"~"Stat"~"p-value",0~0~0~0~0,ff);"";
ff= "#*.*lG"~14~4;
call printfm("mean"~meanL~""~meanSTR,0~1~0~1,ff);"";
call printfm("min"~minL~""~minSTR,0~1~0~1,ff);"";
call printfm("max"~maxL~""~maxSTR,0~1~0~1,ff);"";
call printfm("med"~medL~""~medSTR,0~1~0~1,ff);"";
call printfm("stdev"~stdevL~""~stdevSTR,0~1~0~1,ff);"";
call printfm("mad"~madL~""~madSTR,0~1~0~1,ff);"";"";
call printfm("skew"~skewL~pskewL~skewSTR~pskewSTR,0~1~1~1~1,ff);"";
call printfm("kurt"~kurtL~pkurtL~kurtSTR~pkurtSTR,0~1~1~1~1,ff);"";
call printfm("JB"~JBL~pJBL~JBSTR~pJBSTR,0~1~1~1~1,ff);"";"";
m_r=minc(rows(corrL)|rows(corrSTR));
"correlations";
call printfm(seqa(1,1,m_r)~corrL[1:m_r]~("".*ones(m_r,1))~corrSTR[1:m_r],1~1~0~1,ff);"";"";
"Breusch-Godfrey test of no residual autocorrelation";
ff= "#*.*lG"~14~8;
call printfm("Order"~"Stat"~"p-value"~"Stat"~"p-value",0~0~0~0~0,ff);"";
ff= "#*.*lG"~14~4;
call printfm(seqa(1,1,m_r)~LML[1:m_r,2]~pLML[1:m_r,2]~LMSTR[1:m_r,2]~pLMSTR[1:m_r,2],
    ones(1,5),ff);"";"";
"ARCH test";
ff= "#*.*lG"~14~8;
call printfm("Order"~"Stat"~"p-value"~"Stat"~"p-value",0~0~0~0~0,ff);"";
ff= "#*.*lG"~14~4;
call printfm(seqa(1,1,ARCHq)~ARCHL~pARCHL~ARCHSTR~pARCHSTR,ones(1,5),ff);"";
call printfm("AIC"~AICL~""~AICSTR,0~1~0~1,ff);"";
call printfm("BIC"~BICL~""~BICSTR,0~1~0~1,ff);"";
call printfm("HQ"~HQL~""~HQSTR,0~1~0~1,ff);"";
output off;

/* Save residuals */
fname="lstrres.out";
output file=^fname reset; screen off;
  resid;
output off; screen on;

/* Save values of transition function */
fname="lstrfun.out";
output file=^fname reset; screen off;
  tr_fun;
output off; screen on;

/* Save parameter estimates */
output file="lstrpar.out" reset; screen off;
  param;
output off; screen on;

endp;



/* Make graphs of actual and fitted series, transition function over time,
   residuals and transition function versus transition variable */
proc (0)=THE_FIGURES(fit,resid,tr_fun);
  local f,d,h,sz,temp;

{f,d,h}=UKERNEL(resid,resid,0,ones(rows(resid),1),&k_gauss);

_pdate="";
_pcolor=15;
sz=.15;
_pnum=2;
_ptitlht=sz;
_paxht=sz;
_pnumht=sz;
_ptek="piclstr.tkf";
begwind;
window(2,2,0);
_ptitle="F-function vs. time";
xy(0,tr_fun);
nextwind;
_ptitle="Residuals";
xy(0,resid);
nextwind;
_plctrl = -1;
_ptitle="F-function vs. x_trans";
temp = sortc(_dat[.,_xtri]~tr_fun,1);
xy(temp[.,1],temp[.,2]);
nextwind;
_plctrl=0;
_ptitle="Density residuals";
call VIEWUKNL(resid,f,h,meanc(resid),stdc(resid));
endwind;
endp;


proc(1)=rgr(x,lg);
local regr;

regr=shiftr(x',lg,0)';
retp(regr);
endp;


